<%@ WebHandler Language="C#" Class="UserCommandProcessHandler" %>

using System;
using System.Collections.Generic;
using System.Web;
using System.IO;
using RasterEdge.Imaging.Basic;
using RasterEdge.XDoc.PDF;
using RasterEdge.WDP;
using RasterEdge.WDP.Manager;
using RasterEdge.XDoc.PDF.HTML5Editor;

public class UserCommandProcessHandler : ProcessHandler
{

    /*
         The SDK will use the following url parameter names, please do not use them
          RECommand, src, filepath, restful
    */
    public override PDFWebDocument FileProcess()
    {
        HttpRequest request = this.Context.Request;

        if (!String.IsNullOrEmpty(request.QueryString["yourtarget"]))
        {
            // load file
            String docid = request.QueryString["yourtarget"];
            if (docid == null)
                return null;

            try
            {
                //  Load the target PDF file and import annotations from .xfdf file.
                byte[] dataBytes = preFileLoad(docid);

                return REProcessControl.PageLoadFile(request, dataBytes, new ViewerSettings(docid));
            }
            catch (Exception ex)
            {
                Logger.Log(">>> fail to load the target PDF file. " + ex.Message, LogType.ERROR);
                return null;
            }
        }
        else
        {
            Logger.Log(">>> Unknown load file mode. Load default file defined in Web.config.", LogType.DEBUG);

            // load default file. defined in Web.config
            return REProcessControl.PageLoadFile(request, this.Manager, "", LoadType.Server);
        }
    }

    //  Load the target PDF file (without annotations) and import annotations from the source file (.xfdf).
    private byte[] preFileLoad(String docid)
    {
        //  Plain PDF file without annotation.
        String pdfFilePath = @"C:\\temp\" + docid;
        //  Related annotation file (.xfdf)
        String annotFilePath = @"C:\\temp\" + docid + ".xfdf";

        if (!File.Exists(pdfFilePath))
            throw new ArgumentException("target PDF file does not exist. " + docid);

        //  Import annotations from the .xfdf file to the palin PDF file.
        if (File.Exists(annotFilePath))
        {
            String tmpFilePath = @"C:\\temp\" + docid + ".tmp.pdf";
            try
            {
                List<IPDFAnnot> annots = PDFAnnotHandler.ExtractAnnotsFromXFDF(annotFilePath);
                PDFAnnotHandler.AddAnnotation(pdfFilePath, annots, tmpFilePath);
                return File.ReadAllBytes(tmpFilePath);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (File.Exists(tmpFilePath))
                    File.Delete(tmpFilePath);
            }
        }
        //  Return the plain PDF file if no .xfdf file.
        else
            return File.ReadAllBytes(pdfFilePath);

    }

    //  fid: Task ID.
    //  savePath: The abosolute path of the saved PDF file in the task's output folder.
    //  responseMsg: The response message for saving the current document to the task's output folder.
    //               The string value is the relative path of the output file. "Error" means that the saving process is failed.
    public override void SaveFile_OnServer(String fid, String savePath, String responseMsg)
    {
        //  You can process saved pdf document here, like add watermark, ...
        var documentPath = savePath;
        if (!String.IsNullOrEmpty(documentPath))
        {
            Logger.LogFile(fid, "SaveFileOnServer: output file path " + documentPath, LogType.DEBUG);

            //  To verify the output file.
            BaseDocument document = null;
            if (documentPath.EndsWith(".pdf"))
            {
                // document object includes user modified content
                document = new PDFDocument(documentPath);
            }
            if (document != null)
            {
                PDFDocument pdfDoc = document as PDFDocument;

                try
                {
                    //  Post-process
                    PDFMetadata metadata = pdfDoc.GetDescription();
                    metadata.Producer = "RasterEdge EdgePDF";
                    pdfDoc.SetDescription(metadata);
                }
                catch (Exception ex)
                {
                    //  Process error code, and return error information here
                    Logger.LogFile(fid, ex.Message + ". fail to do post-process.", LogType.ERROR);
                }

                //  Get the upload information of the file
                DocUploadInfo docinfo = FileManager.getUploadinfoByFid(fid);
                //  Get your file open url parameters value, if needed.
                String paraFilepathValue = docinfo.GetRequestParameters("yourtarget");
                if (!String.IsNullOrEmpty(paraFilepathValue))
                {
                    try
                    {
                        //  Update the plain PDF file and .xfdf file.
                        saveDocument(pdfDoc, @"C:\\temp\" + paraFilepathValue);
                    }
                    catch (Exception ex)
                    {
                        //  Process error code, and return error information here
                        Logger.LogFile(fid, ex.Message + ". fail to save file to server.", LogType.ERROR);
                    }
                }
                else
                {
                    Logger.LogFile(fid, "no 'yourtarget' in the HTTPRequest.", LogType.INFO);
                }
            }
            else
            {
                Logger.LogFile(fid, "output PDF file is invalid.", LogType.ERROR);
            }
        }
        else
        {
            Logger.LogFile(fid, "fail to save file to server. output file path is null or empty.", LogType.ERROR);
        }
    }

    //  Save and update the original plain PDF file and annotation source file (.xfdf).
    private void saveDocument(PDFDocument pdfDoc, String pdfFilePath)
    {
        String tmpFilePath = pdfFilePath + ".tmp.pdf";
        try
        {
            //  Create and update the annotation file (.xfdf).
            pdfDoc.Save(tmpFilePath);
            String annotFilePath = pdfFilePath + ".xfdf";
            PDFDocument.ExportXFDFDocument(tmpFilePath, annotFilePath);

            //  Create and update the plain PDF file.
            PDFDocument doc = new PDFDocument(tmpFilePath);
            List<IPDFAnnot> annots = PDFAnnotHandler.GetAllAnnotations(doc);
            PDFAnnotHandler.DeleteAnnotation(doc, annots);
            doc.Save(pdfFilePath);
        }
        catch (Exception ex)
        {
            throw ex;
        }
        finally
        {
            if (File.Exists(tmpFilePath))
                File.Delete(tmpFilePath);
        }
    }
}